#include "DllHeader.h"
#include <stdio.h>
#include <stdlib.h>

//Global variables
double kp, ki, ts;

typedef struct
{
	FILE* fp;
} MyData;

MyData* data;

//Map input and output parameters to meaningful names
#define e aState->inputs[0]
#define i aState->states[0]
#define y aState->outputs[0]


DLLEXPORT void plecsSetSizes(struct SimulationSizes* aSizes)
{
	aSizes->numInputs = 1;
	aSizes->numOutputs = 1;
	aSizes->numStates = 1;
	aSizes->numParameters = 3; //number of user parameters passed in
}


//This function is automatically called at the beginning of the simulation
DLLEXPORT void plecsStart(struct SimulationState* aState)
{
	//Example error message box
	if (aState->parameters[0] < 0)
		aState->errorMessage = "kp cannot be less than 0";
	kp = aState->parameters[0];
	ki = aState->parameters[1];
	ts = aState->parameters[2];

	//Allocate custom data structure
	data = (MyData*)malloc(sizeof(MyData));
	aState->userData = data;

	//Open File
	errno_t err= fopen_s(&data->fp,"Z:\\Documents\\Work\\projects\\support\\Forum_CScript_Saving_20210325\\dll_file_out\\results.txt", "w+");
	if (err==0)
	{
		fprintf(data->fp, "Time(us)\tInteg\toutput\n");
	}
	else 
	{
		aState->errorMessage = "Cannot open file for writing.";
	}

}


//This function is automatically called every sample time
//output is written to DLL output port after the output delay
DLLEXPORT void plecsOutput(struct SimulationState* aState)
{
	i = i + ts * e;
	y = kp * e + ki * i;


	fprintf(data->fp, "%6.1f\t\t%4.4f\t%4.4f\n", 1.0e6 * aState->time, i, y);
	
}

// Called when the simulation is finished, even when an error occured.       
DLLEXPORT void plecsTerminate(struct SimulationState* aState)
{
	if (data->fp != NULL) 
	{
		fclose(data->fp);
	}
	free(aState->userData);
}
